//	DataStreams.cpp

#include <DataStreams.h>

#include <string.h>
#include <OS.h>
#include <stdlib.h>
#include <stdio.h>




#pragma mark --- DFileStream ---

DFileStream::DFileStream(
	BFile * file,
	off_t offset)
{
	fFile = file;
	fOffset = offset;
	file->Seek(offset, SEEK_SET);
}


DFileStream::~DFileStream()
{
	if (fDispose) {
		delete fFile;
	}
}


ssize_t
DFileStream::Read(
	void * buffer,
	ssize_t & size)
{
	size = fFile->Read(buffer, size);
	if (size < 0)
		return size;
	if (!size)
		return B_ERROR;
	return 0;
}


ssize_t
DFileStream::Write(
	const void * buffer,
	ssize_t size)
{
	ssize_t err = fFile->Write(buffer, size);
	if (err < 0)
		return err;
	return B_OK;
}


ssize_t
DFileStream::Seek(
	off_t position)
{
	if (position < 0)
		return B_ERROR;
	if (position > Size())
		return B_BAD_VALUE;
	off_t err = fFile->Seek(position+fOffset, 0);
	if (err < 0)
		return err;
	return B_OK;
}


off_t
DFileStream::Position()
{
	off_t err = fFile->Seek(0, SEEK_CUR);
	if (err >= 0)
		err -= fOffset;
	return err;
}


off_t
DFileStream::Size()
{
	off_t pos = fFile->Seek(0, SEEK_CUR);
	fFile->Seek(0, SEEK_END);
	off_t size = fFile->Seek(0, SEEK_CUR);
	if (pos >= 0)
		fFile->Seek(pos, SEEK_SET);
	return size-fOffset;
}


ssize_t
DFileStream::SetSize(
	off_t size)
{
	off_t pos = fFile->Seek(0, SEEK_CUR);
	if (pos > size+fOffset) {
		pos = size+fOffset;
		fFile->Seek(pos, SEEK_SET);
	}
	ssize_t err = fFile->SetSize(size+fOffset);
	if (pos >= 0)
		fFile->Seek(pos, SEEK_SET);
	return err;
}





#pragma mark --- DMallocStream ---

DMallocStream::DMallocStream(
	void * ptr,
	ssize_t size,
	ssize_t offset)
{
	fPtr = ptr;
	if (!fPtr)
		fPtr = malloc(size+offset+1);	//	make sure we never get NULL
	fPhysSize = size+offset;
	fLogSize = size;
	fLogPos = 0;
	fOffset = offset;
}


DMallocStream::~DMallocStream()
{
	if (fDispose && fPtr) free(fPtr);
}


ssize_t
DMallocStream::Read(
	void * buffer, 
	ssize_t & size)
{
	if (fLogPos >= fLogSize) {
		size = 0;
		return B_ERROR;
	}
	if (size > fLogSize-fLogPos)
		size = fLogSize-fLogPos;
	memcpy(buffer, ((char *)fPtr)+fOffset+fLogPos, size);
	fLogPos += size;
	return 0;
}


ssize_t
DMallocStream::Write(
	const void * buffer,
	ssize_t size)
{
	if (size+fLogPos+fOffset > fPhysSize) {
		ssize_t newSize = fPhysSize + (fPhysSize-fOffset);
		if (newSize < size+fLogPos+fOffset)
			newSize = size+fLogPos+fOffset+1024;
		void *newPtr = realloc(fPtr, newSize);
		if (!newPtr)
			return B_NO_MEMORY;
		fPtr = newPtr;
		fPhysSize = newSize;
	}
	memcpy(((char *)fPtr)+fOffset+fLogPos, buffer, size);
	fLogPos += size;
	if (fLogPos > fLogSize)
		fLogSize = fLogPos;
	return 0;
}


ssize_t
DMallocStream::Seek(
	off_t position)
{
	if (position < 0 || position > fLogSize)
		return B_ERROR;
	fLogPos = position;
	return B_NO_ERROR;
}


off_t
DMallocStream::Position()
{
	return fLogPos;
}


off_t 
DMallocStream::Size()
{
	return fLogSize;
}


ssize_t
DMallocStream::SetSize(
	off_t size)
{
	if (fLogSize != size) {
		if (size > fPhysSize-fOffset) {
			void *newPtr = realloc(fPtr, size+fOffset);
			if (!newPtr)
				return B_NO_MEMORY;
			fPtr = newPtr;
			fPhysSize = size+fOffset;
		}
		fLogSize = size;
		if (fLogPos > size)
			fLogPos = size;
	}
	return B_NO_ERROR;
}


const void *
DMallocStream::Peek(
	ssize_t & outOffset,
	ssize_t & outSize)
{
	outOffset = fOffset;
	outSize = fLogSize;
	return fPtr;
}




#pragma mark --- DAreaStream ---

DAreaStream::DAreaStream(
	area_id area,
	ssize_t size,
	ssize_t offset)
{
	fPhysSize = size+offset;
	fLogSize = size;
	fLogPos = 0;
	fOffset = offset;
	fArea = area;
	fPtr = NULL;
	if (!area) {
		ssize_t s = (size+offset+4095)&~4095;	//	page size is 4096
		if (s < 4096) s = 4096;
		area = create_area("DAreaStream", &fPtr, B_ANY_ADDRESS,
				s, B_NO_LOCK, B_READ_AREA | B_WRITE_AREA);
		if (area < 0)
			return;
		fArea = area;
		fPhysSize = s;
	}
	area_info info;
	get_area_info(fArea, &info);
	fPtr = info.address;
}


DAreaStream::~DAreaStream()
{
	if (fDispose && (fArea > 0))
		delete_area(fArea);
}


ssize_t
DAreaStream::Read(
	void * buffer, 
	ssize_t & size)
{
	if (fLogPos >= fLogSize) {
		size = 0;
		return B_ERROR;
	}
	if (size > fLogSize-fLogPos)
		size = fLogSize-fLogPos;
	memcpy(buffer, ((char *)fPtr)+fOffset+fLogPos, size);
	fLogPos += size;
	return 0;
}


ssize_t
DAreaStream::Write(
	const void * buffer,
	ssize_t size)
{
	if (size+fLogPos+fOffset > fPhysSize) {
		ssize_t newSize = fPhysSize + (fPhysSize-fOffset);
		if (newSize < size+fLogPos+fOffset)
			newSize = size+fLogPos+fOffset+1024;
		newSize = (newSize+4095)&~4095;
		ssize_t err = resize_area(fArea, newSize);
		if (err)
			return err;
		area_info info;
		get_area_info(fArea, &info);
		fPtr = info.address;
		fPhysSize = newSize;
	}
	memcpy(((char *)fPtr)+fOffset+fLogPos, buffer, size);
	fLogPos += size;
	if (fLogPos > fLogSize)
		fLogSize = fLogPos;
	return 0;
}


ssize_t
DAreaStream::Seek(
	off_t position)
{
	if (position < 0 || position > fLogSize)
		return B_ERROR;
	fLogPos = position;
	return B_NO_ERROR;
}


off_t
DAreaStream::Position()
{
	return fLogPos;
}


off_t 
DAreaStream::Size()
{
	return fLogSize;
}


ssize_t
DAreaStream::SetSize(
	off_t size)
{
	if (fLogSize != size) {
		if (size > fPhysSize+fOffset) {
			ssize_t newSize = (size+fOffset+4095)&~4095;
			ssize_t err = resize_area(fArea, newSize);
			if (err)
				return err;
			area_info info;
			get_area_info(fArea, &info);
			fPtr = info.address;
			fPhysSize = newSize;
		}
		fLogSize = size;
		if (fLogPos > size)
			fLogPos = size;
	}
	return B_NO_ERROR;
}


const void *
DAreaStream::Peek(
	ssize_t & outOffset,
	ssize_t & outSize)
{
	outOffset = fOffset;
	outSize = fLogSize;
	return fPtr;
}

